package Site::Page;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven. All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use PObject;
@ISA = qw( PObject );

use strict;

require( "Site/cookie.lib" );

use DB_File;
use URI::Escape;
use HTML::Entities;

%Site::Page::AUTOFIELDS = ( TITLE => "Untitled",
				SUBTITLE => undef,
				SESSION => undef,
				OPTIONS => undef,
				COOKIES => undef,
				USESESSIONS => 0,
				SESSIONID => undef,
				SESSIONFILE => '/tmp/default_session.%s.db',

				INITIALIZED => undef );

my %OPTS;
my( $CMD, $A, $B );

# This will automagically slurp in the web data
if( $main::ENV{ REQUEST_METHOD } eq "POST" )
{
    read STDIN, $CMD, $main::ENV{ CONTENT_LENGTH };
}
else
{
    $CMD = $main::ENV{ QUERY_STRING };
}

# Fix spaces
$CMD =~ s/\+/ /g;

for ( split( /&/, $CMD ) )
{
    s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;

    ( $A, $B ) = $_ =~ /^(.*?)=(.*)/s;

    # All keys are lowercase
    $OPTS{ lc( $A ) } = $B;
}



# Handles std look and feel of pages
sub Initialize
{
	# Look up saved state, if any
	my $Self = shift;

	my %Session;
	my $SessionName;

	GetCookies();

	if( $Self -> UseSessions() )
	{
		if( exists( $Site::Page::Cookies{ sid } ) )
		{
			$SessionName = $Self -> _GenerateSessionName( $Site::Page::Cookies{ sid } );
			$Self -> SessionID( $Site::Page::Cookies{ sid } );
		}
		else
		{
			$Self -> SessionID( $$ . time() );
			SetCookies( 'sid', $Self -> SessionID() );
			$SessionName = $Self -> _GenerateSessionName( $$ . time() );
		}

		$Self -> _dmesg( "Tying to session: $SessionName", 3 );
		tie %Session, 'DB_File', $SessionName;

		# Put in some standard infos
		$Session{ lastaccessed } = time();

		$Session{ ip } = $main::ENV{ "REMOTE_ADDR" };

		$Session{ lasturi } = $Session{ thisuri };
		$Session{ thisuri } = $main::ENV{ "REQUEST_URI" };
		if( $Session{ lasturi } ne $Session{ thisuri } )
		{
			$Session{ lastdistincturi } = $Session{ lasturi };
		}

		$Self -> { SESSION } = \%Session;
	}

	# Save all the hashes
	$Self -> { OPTIONS } = \%OPTS;
	$Self -> { COOKIES } = \%Site::Page::Cookies;
}

sub Start
{
	my $Self = shift;

	my $Title = $Self -> Title();
	my $Secondary = $Self -> SubTitle() || "No Sub Title";
  
    print <<EOF;
Content-type: text/html


<HTML>
  <HEAD>
    <TITLE>$Title - $Secondary</TITLE>
  </HEAD>
  
  <BODY>

EOF
}


sub End
{
    print <<EOF;
    </BODY>

</HTML>
EOF

}

sub SetCookie
{
	my $Self = shift;
	my %Hash = @_;

	for( keys( %Hash ) )
	{
		SetCookies( $_, $Hash{ $_ } );
	}
}

sub Redirect
{
	my $Self = shift;

	print "Location: " . shift();
	print "\n\n";
	exit( 0 );
}

sub _GenerateSessionName
{
	my $Self = shift;

	my $SessionID = shift;

	return( sprintf( $Self -> SessionFile(), $SessionID ) );
}

sub EscapeForURI
{
	my $Self = shift;
	my $Text = uri_escape( shift() );
	# Doesn't escape out &?
	$Text =~ s/&/\%26/g;
	return( $Text );
}

sub EscapeURI
{
	shift() -> EscapeForURI( shift() );
}

sub EncodeEntities
{
	my $Self = shift;

	return( encode_entities( shift() ) );
}

sub DecodeEntities
{
	my $Self = shift;
	return( decode_entities( shift() ) );
}

1;
 
