package Site::SQLFieldBase;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven. All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA );

use PObject;
@ISA = qw( PObject );

use Site::Page;

%Site::SQLFieldBase::AUTOFIELDS = (

# This contributes to the html field name
			NAME => undef,
# SQL field this is bound to
			BOUNDTO => undef,
# Label on the HTML form to be used
			LABEL => undef,

# Field type
#  text
			TYPE => undef,

# Enabled means the field can be edited in the browser
			ENABLED => 1,

# Locked means the field will not be part of the update SQL,
# it's still present in the select part.
			LOCKED => 0,

# Visible controls if the field is visible on the browser
			VISIBLE => 1,

# If the field is not bound to an SQL field, this should be set
			VALUE => undef,
# Default value for new records.  Overrides value in case of conflict
			DEFAULT => undef,

# Set by the container object, the container should provide an
# _HTMLName function for creating the HTML names
			CONTAINER => undef
		);



# For the sql field names, the user class should weed out undef
# vars because they are not relevant to the task at hand.
sub SQLSelect
{
	my $Self = shift;

	return( $Self -> BoundTo() );
}

sub SQLUpdate
{
	my $Self = shift;

	# If enabled and not locked....
	if( ! $Self -> Locked() ) 
	{
		return( $Self -> BoundTo() );
	}

	return( undef );
}

# This really should be overriden by the final class modules
sub _HTMLField
{
	my $Self = shift;
	return( "Class did not override field html" );
}

sub _HTMLLabel
{
	my $Self = shift;

	if( $Self -> Label() )
	{
		return( $Self -> Label() );
	}elsif( $Self -> BoundTo() )
	{
		return( $Self -> BoundTo() );
	}elsif( $Self -> Name() )
	{
		return( $Self -> Name() );
	}
}

sub SetDefault
{
	my $Self = shift;

	$Self -> { VALUE } = $Self -> Default();
}

# These are the ACTUAL methods to call by any user classes
sub HTMLField
{
	my $Self = shift;

	my $RVal;

	if( $Self -> Visible() )
	{
		if( $Self -> Enabled() )
		{
			$RVal = $Self -> _HTMLField();
		}else
		{
			# Not enabled, no editing, just display it
			$RVal = $Self -> _HTMLDisplayValue();
		}
	}

	# If I'm not locked I must include something
	if( ( ( ! $Self -> Locked() ) &&
		( $Self -> Visible() ) &&
		( ! $Self -> Enabled() ) ) ||
		( ! $Self -> Visible() ) )
	{
		# Just a hidden me
		$RVal .= '<INPUT TYPE=HIDDEN NAME=' . 
			$Self -> _HTMLName() .
			' VALUE="' .
			$Self -> _HTMLValue() .
			'">';
	}

	return( $RVal );
}

sub HTMLLabel
{
	my $Self = shift;
	if( $Self -> Visible() )
	{
		return( $Self -> _HTMLLabel() );
	}
	return( undef );
}

sub _HTMLValue
{
	# Returns the value HTML'ized
	my $Self = shift;
	return( Site::Page -> EncodeEntities( $Self -> Value() ) );
}

# If the value shown is different than the value used,
# then this should be overriden in the subclass
sub _HTMLDisplayValue
{
	my $Self = shift;
	return( $Self -> _HTMLValue() );
}

sub HTMLDisplay
{
	return( shift() -> _HTMLDisplayValue() );
}

sub _HTMLName
{
	# Returns the fully qualified field name
	my $Self = shift;
	if( $Self -> Container() )
	{
		return( $Self -> Container() -> _HTMLName() .
			"." .
			$Self -> Name() );
	}
	return( $Self -> Name() );
}

sub SetValue
{
	my $Self = shift;

	$Self -> { VALUE } = shift();
}

sub Disabled
{
	# Shorthand for locked=1/enabled=0
	my $Self = shift;
	if( @_ )
	{
		$Self -> Locked( 1 );
		$Self -> Enabled( 0 );
	}else
	{
		$Self -> Locked( 0 );
		$Self -> Enabled( 1 );
	}
}

sub disabled
{
	shift() -> Disabled( @_ );
}

1;
