package Site::SQLForm;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven. All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA %AUTOFIELDS $AUTOSETTINGS );

use PObject;
@ISA = qw( PObject );

use Site::Page;
use Site::SQLField;
use Site::Template;

$AUTOSETTINGS = 1;
%AUTOFIELDS = (
			FORMACTION => $::ENV{ SCRIPT_NAME },
			TABLE => undef,
			PKEYS => undef,
			NAMEDFIELDS => undef,
			OPTS => undef,
			DBH => undef,
			TEMPLATE => undef,
			DELETETEMPLATE => undef,
			HTMLFIELDS => undef,
			BOUNDFIELDS => undef,
			DELETEBUTTON => 'Delete',
			ADDBUTTON => 'Add',
			EDITBUTTON => 'Edit',
			DELETEYESBUTTON => 'Yes',
			DELETENOBUTTON => 'No',
			MESSAGE => undef,
			PKEYVALS => undef,
			CARRY => undef
	);

sub OBJ_NEW
{
	my $Self = shift;

	$Self -> { NAMEDFIELDS } = {};
	$Self -> { PKEYS } = [];
	$Self -> { HTMLFIELDS } = [];
	$Self -> { CARRY } = {};
	$Self -> { BOUNDFIELDS } = {};

	if( ! defined( $Self -> { OPTS } ) )
	{
		$Self -> { OPTS } = {};
	}

	return( $Self );
}

sub AddField
{
	my $Self = shift;
	my %Attr = @_;

	$Self -> { NAMEDFIELDS } -> { $Attr{ name } } =
		Site::SQLField -> new( %Attr );
	push( @{ $Self -> { HTMLFIELDS } }, $Self -> NamedFields() -> { $Attr{ name } } );

	$Self -> { NAMEDFIELDS } -> { $Attr{ name } } -> Container( $Self );

	return( $Self -> { NAMEDFIELDS } -> { $Attr{ name } } );
}

sub AddFields
{
	my $Self = shift;

	# Just add the fields
	for ( @_ )
	{
		$Self -> AddField( %{ $_ } );
	}
}

sub Handle
{
	# Return true if handled something, false if not
	# Will also set BADFORM if I'm bad.  :)
	my $Self = shift;

	# Check for PKEY values and fill them in if applicable
	$Self -> _FillPKeys();
	$Self -> _MapBoundFields();

	my $Do = $Self -> _WhatToDo();
	my $MagicQ;
	my @FieldNames = $Self -> _ModFields();
	my @Values; # = $Self -> _Values();

	#print "$Do<BR>";

	# Gather fields values
	#for( keys( %{ $Self -> Opts() } ) )
	#{
	#	print $_ . ": " . $Self -> Opts() -> { $_ } . "<BR>";
	#}

	# Do?
	if( ! grep( $Do eq $_, 'UPDATE',
				'ADD',
				'DELETE' ) )
	{
		return( undef );
	}

	for( @FieldNames )
	{
		#print $Self -> GetField( $_ ) -> _HTMLName();
		push( @Values, $Self -> Opts() -> 
			{ $Self -> GetFieldBoundTo( $_ ) -> _HTMLName() } );
	}

	if( $Do eq "UPDATE" )
	{
		# Perform an update query
		$MagicQ = 'update ' .
				$Self -> Table() .
				' set ' .
				join( ", ", map( $_ . "=?",
					$Self -> _ModFields() ) ) .
				' where ' .
				join( " AND ", map( $_ . "=?",
					@{ $Self -> PKeys() } ) );

		# Add PKEYS to the values array
		for( @{ $Self -> PKeys } )
		{
			push( @Values, $Self -> Opts() -> {
					$Self -> _HTMLName() . '.pkeys.' . $_ }
				);
		}
	}elsif( $Do eq "ADD" )
	{
		# Perform an insert query
		$MagicQ = 'insert into ' .
			$Self -> Table() .
			' ( ' .
			join( ", ", $Self -> _ModFields() ) .
			' ) values ( ' .
			join( ", ", map( '?', $Self -> _ModFields() ) ) .
			' ) ';
	}elsif( $Do eq "DELETE" )
	{
		# Perform deleteooo
		$MagicQ = 'delete from ' .
				$Self -> Table() . 
				' where ' .
				join( " AND ", map( $_ . "=?",
						@{ $Self -> PKeys() } ) );
		@Values = ();
		for( @{ $Self -> PKeys } )
		{
			push( @Values, $Self -> Opts() -> {
					$Self -> _HTMLName() . '.pkeys.' . $_ }
				);
		}
	}

	#print $MagicQ;
	#print "(" . join( ", ", @Values ) . ")";

	# Do it!
	my $Status = $Self -> DBH() -> do( $MagicQ, undef, @Values );

	if( ! defined( $Status ) )
	{
		# Baddy!
		$Self -> { BADFORM } = 1;
		$Self -> AddMessage( "An SQL error occurred: '" .
				$Self -> DBH() -> errstr() . "'" );
	}

	return( $Do );
}

sub Display
{
	my $Self = shift;

	# Decide what to do based on vars
	$Self -> _FillFields();
	#print $Self -> _WhatForm();

	if( $Self -> _WhatForm() eq "DELETE" )
	{
		$Self -> _DisplayDelete();
	}else
	{
		$Self -> _DisplayForm();
	}
}

sub _WhatToDo
{
	# Look at the opts and decide what to do
	my $Self = shift;
	my $ActVal = $Self -> Opts() -> { $Self -> _Action() };
	my $ButVal = $Self -> Opts() -> { $Self -> _Button() };

	#print $ActVal . "::" . $ButVal . "<BR>";
	if( $ButVal eq $Self -> DeleteButton() )
	{
		# If they pushed delete then ask if they are sure
		return( "ASKDELETE" );
	}elsif( $ActVal eq "DELETE" )
	{
		# They said yes?
		if( $ButVal eq $Self -> DeleteYesButton() )
		{
			return( "DELETE" );
		}else
		{
			# Nope
			return( "NODELETE" )
		}
	}elsif( $ActVal eq "UPDATE" )
	{
		return( "UPDATE" );
	}elsif( $ActVal eq "ADD" )
	{
		return( "ADD" );
	}elsif( $Self -> PKeyVals() ) ### FIX ME
	{
		return( "ASKUPDATE" );
	}else
	{
		# Nothing means askadd
		return( "ASKADD" );
	}
}

sub _WhatForm
{
	# Just a logic filter for the above
	my $Self = shift;
	my $Do = $Self -> _WhatToDo();

	# This will tell what form to display and the method it should do
	if( grep( $_ eq $Do, 'ASKADD',
				'ADD',
				'UPDATE',
				'DELETE',
				'NODELETE' ) )
	{
		return( "ADD" );
	}elsif( $Do eq 'ASKUPDATE' )
	{
		return( "UPDATE" );
	}elsif( $Do eq "ASKDELETE" )
	{
		# Munge the PKEYS out of the query
		$Self -> PKeyVals( [] );
		for( @{ $Self -> PKeys } )
		{
			push( @{ $Self -> PKeyVals() }, $Self -> Opts() -> {
					$Self -> _HTMLName() . '.pkeys.' . $_ }
				);
		}
		return( "DELETE" );
	}

	return( undef );
				
}

sub _FillFields
{
	# Call proper filler based on action to take
	my $Self = shift;
	my $Do = $Self -> _WhatForm();

	# If askadd or just updated or just deleted, then fill defaults
	if( $Do eq 'ADD' )
	{
		$Self -> _FillFieldsAdd();
	}elsif( $Do eq "UPDATE" )
	{
		$Self -> _FillFieldsUpdate();
	}elsif( $Do eq "DELETE" )
	{
		$Self -> _FillFieldsUpdate();
	}
}

sub _FillFieldsUpdate
{
	# This will get values from an SQL query
	my $Self = shift();

	my @PKeys = @{ $Self -> PKeys() };
	my @Fields = $Self -> _SelFields();
	my @Values;

	# Generate the select query
	my $MagicQ = 'select ' . join( ", ", @Fields ) .
		' from ' . $Self -> Table() .
		' where ' .
		join( " AND ", map( $_ . "=?", @PKeys ) );

	my $sth = $Self -> DBH() -> prepare( $MagicQ );
	$sth -> execute( @{ $Self -> PKeyVals() } );

	my $Ref = $sth -> fetchrow_hashref();

	if( ! defined( $Ref ) )
	{
		$Self -> AddMessage( "Row now found" );
		# Should break it down a bunch here
		return( undef );
	}
	for( keys( %{ $Ref } ) )
	{
		$Self -> GetFieldBoundTo( $_ ) -> SetValue( $Ref -> { $_ } );
	}
}

sub _FillFieldsAdd
{
	my $Self = shift;

	for( @{ $Self -> HTMLFields() } )
	{
		$_ -> SetDefault();
	}
}

sub _FillFieldsBad
{
	# This will get values from the environment
}

sub _DisplayForm
{
	my $Self = shift;

	my $Ret;

	# Performs the actual form display
	if( ! $Self -> Template() )
	{
		$Self -> Template( Site::Template -> new() );
		$Self -> Template() -> Template( $Self -> _DefaultTemplate() );
	}

	$Self -> Template() -> RegisterDefault( $Self );
	$Self -> Template() -> Display();
}

sub _DisplayDelete
{
	my $Self = shift;
	if( ! $Self -> DeleteTemplate() )
	{
		$Self -> DeleteTemplate( Site::Template -> new() );
		$Self -> DeleteTemplate() ->
			Template( $Self -> _DefaultDeleteTemplate() );
	}

	$Self -> DeleteTemplate() -> RegisterDefault( $Self );
	$Self -> DeleteTemplate() -> Display();
}

sub _DefaultTemplate
{
	# Return a default template
	return( qq{<t:var name=HTMLBegin/>
<PRE>
<t:var name=Message/>
</PRE>
<TABLE BORDER=0>
<t:loop name=HTMLFields>
<TR>
<TD><t:iftrue name=HTMLLabel><B><t:var name=HTMLLabel/>:</B></t:iftrue></TD>
<TD><t:var name=HTMLField/></TD>
</TR>
</t:loop>
</TABLE>
<t:var name=HTMLAdd/>
<t:var name=HTMLEdit/>
<t:var name=HTMLDelete/>
<t:var name=HTMLEnd/>
} );

}

sub _DefaultDeleteTemplate
{
	return( qq{<t:var name=HTMLBegin/>
Delete it?

<TABLE BORDER=0>
<t:loop name=HTMLFields>
<TR>
<TD><t:iftrue name=HTMLLabel><B><t:var name=HTMLLabel/>:</B></t:iftrue></TD>
<TD><t:var name=HTMLDisplay/></TD>
</TR>
</t:loop>
</TABLE>

<t:var name=HTMLDeleteYes/>
<t:var name=HTMLDeleteNo/>
<t:var name=HTMLEnd/>
} );
}

sub _HTMLName
{
	my $Self = shift;
	return( join( ".", $Self -> Table(), @{ $Self -> PKeys() } ) );
}

sub _Action
{
	# The name of my action
	my $Self = shift;
	return( $Self -> _HTMLName() . ".__.action" );
}

sub HTMLBegin
{
	my $Self = shift;
	my $Action = $Self -> FormAction();
	my $Do = $Self -> _WhatForm();

	my $Ret = qq{<FORM ACTION=$Action METHOD=GET>
} . '<INPUT TYPE=HIDDEN NAME=' . $Self -> _Action() .
	' VALUE="' . $Do . '">';

	if( $Do ne "ADD" )
	{
		# Add pkey vals
		my @Vals = @{ $Self -> PKeyVals() };
		for( @{ $Self -> PKeys() } )
		{
			$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
				$Self -> _HTMLName() .
				'.pkeys.' .
				$_ .
				' VALUE="' .
				Site::Page ->
					EncodeEntities( shift( @Vals ) ) .
				'">';
		}
	}

	# Spew forth the carrables!
	for ( keys( %{ $Self -> Carry() } ) )
	{
		$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
			$_ .
			' VALUE="' .
			Site::Page ->
				EncodeEntities( $Self -> Carry() -> { $_ } ) .
			'">';
	}

	return( $Ret );
}


sub HTMLEnd
{
	return( qq{</FORM>} );
}

sub HTMLLabel
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> GetField( $Fname ) -> HTMLLabel() );
}

sub HTMLField
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> GetField( $Fname ) -> HTMLField() );
}

sub GetField
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> NamedFields() -> { $Fname } );
}

sub HTMLAdd
{
	# The add button, if applicable
	my $Self = shift;

	if($Self -> _WhatForm() eq "ADD" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button() .
			' VALUE="' . 
			$Self -> AddButton() .
			'">' );
	}
	return( undef );
}

sub HTMLEdit
{
	# The edit button
	my $Self = shift;
	if( $Self -> _WhatForm() eq "UPDATE" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button() .
			' VALUE="' . 
			$Self -> EditButton() .
			'">' );
	}

	return( undef );
}

sub HTMLDelete
{
	# The delete button
	# Only on the update form
	my $Self = shift;

	if( $Self -> _WhatForm() eq "UPDATE" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteButton() . '">' ); 
	}
	return( undef );
}

sub HTMLDeleteYes
{
	my $Self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteYesButton() . '">' ); 
}

sub HTMLDeleteNo
{
	my $Self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteNoButton() . '">' ); 
}

sub HTMLDisplay
{
	my $Self = shift;
	return( $Self -> GetField( shift() ) -> DisplayValue() );
}

sub _Button
{
	my $Self = shift;

	return( $Self -> _HTMLName() . ".__.button" );
}

sub _SelFields
{
	my $Self = shift;
	# Return the field names of the fields to mod

	return( grep( /^.+$/, map( $_ -> SQLSelect(),
				@{ $Self -> { HTMLFIELDS } } ) ) );

}

sub _ModFields
{
	my $Self = shift;
	# Return the field names of the fields to mod

	return( grep( /^.+$/, map( $_ -> SQLUpdate(),
				@{ $Self -> { HTMLFIELDS } } ) ) );

}

sub AddMessage
{
	my $Self = shift;
	$Self -> { MESSAGE } .= shift() . "\n";
}

sub _FillPKeys
{
	my $Self = shift;
	my @Arr;
	my $IsSet = 0;

	for( @{ $Self -> _HTMLPKeys() } )
	{
		push( @Arr, $Self -> Opts() -> { $_ } );
		if( defined( $Arr[ -1 ] ) )
		{
			$IsSet = 1;
		}
	}

	if( $IsSet )
	{
		$Self -> { PKEYVALS } = \@Arr;
	}
}

sub _HTMLPKeys
{
	# This will be configurable later, for now the pkeys are not fully
	# qualified
	my $Self = shift;

	return( $Self -> PKeys() );
}

sub AddCarry
{
	# Add vars to the carryo
	my $Self = shift;
	my %Carry = @_;

	for( keys( %Carry ) )
	{
		$Self -> Carry() -> { $_ } = $Carry{ $_ };
	}
}

sub PKey
{
	my $Self = shift;
	$Self -> PKeys( [ shift() ] );
}

sub pkey
{
	shift() -> PKey( @_ );
}

sub _MapBoundFields
{
	my $Self = shift;

	for( @{ $Self -> HTMLFields() } )
	{
		if( $_ -> BoundTo() )
		{
			$Self -> BoundFields() -> { $_ -> BoundTo() } = $_;
		}
	}
}

sub GetFieldBoundTo
{
	my $Self = shift;
	return( $Self -> BoundFields() -> { shift() } );
}

sub DefaultDBH
{
        shift();

        $AUTOFIELDS{ DBH } = shift();
}

sub DefaultOpts
{
        shift();

        $AUTOFIELDS{ OPTS } = shift();
}

1;
